@@ -20,21 +20,34 @@
  * OSD routines.
  */
 
+/* XXX gtk+ 2.3 GtkOptionMenu, Gnome font picker, color picker */
+/* gdk_input_add/remove */
+#undef GTK_DISABLE_DEPRECATED
+#undef GNOME_DISABLE_DEPRECATED
+#undef GDK_DISABLE_DEPRECATED
+
+#include "site_def.h"
+
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#  include "config.h"
 #endif
 
-#ifdef HAVE_LIBZVBI
-
 #include <gnome.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #define ZCONF_DOMAIN "/zapping/options/osd/"
 #include "zconf.h"
 #include "zmisc.h"
 #include "osd.h"
-#include "../common/math.h"
-#include "../common/ucs-2.h"
+#include "remote.h"
+#include "common/math.h"
+#include "properties.h"
+#include "interface.h"
+#include "globals.h"
+#include "subtitle.h"
+
+#ifndef OSD_TEST
+#define OSD_TEST 0
+#endif
 
 #define MAX_COLUMNS 48 /* TTX */
 #define MAX_ROWS 26 /* 25 for TTX plus one for OSD */
@@ -90,7 +103,7 @@ paint_piece			(piece		*p,
 }
 
 static gboolean
-expose				(GtkWidget	*da,
+expose				(GtkWidget	*da _unused_,
 				 GdkEventExpose	*event,
 				 piece		*p)
 {
@@ -130,8 +143,9 @@ set_piece_geometry		(piece		*p)
     {
       dest_x = (gint)(cx + p->x * cw);
       dest_y = (gint)(cy + p->y * ch);
-      dest_sw = dest_w = (gint)(p->w * cw);
-      dest_h =(gint)(p->h * ch);
+      dest_w = (gint)(p->w * cw);
+      dest_h = (gint)(p->h * ch);
+      dest_sw = dest_w;
     }
 
   if (osd_window && ((!p->scaled)  ||
@@ -140,7 +154,7 @@ set_piece_geometry		(piece		*p)
     {
       if (p->scaled)
 	{
-	  gdk_pixbuf_unref(p->scaled);
+	  g_object_unref (G_OBJECT (p->scaled));
 	  p->scaled = NULL;
 	}
       if (dest_h > 0)
@@ -197,7 +211,7 @@ set_piece_geometry		(piece		*p)
 		  /* We should always draw the whole p->scaled, no
 		     more, no less. Otherwise sth is severely b0rken. */
 		  g_assert(scaled_x == dest_w);
-		  gdk_pixbuf_unref(canvas);
+		  g_object_unref (G_OBJECT (canvas));
 		}
 	    }
 	}
@@ -278,6 +292,7 @@ push_window(GtkWidget *window)
 }
 
 /* Unget all windows in the stack */
+static void clear_stack (void) __attribute__ ((unused));
 static void
 clear_stack(void)
 {
@@ -314,9 +329,11 @@ pop_window(void)
 static void
 add_piece			(GdkPixbuf	*unscaled,
 				 GtkWidget	*da,
-				 int col,	int row,
-				 int width,	int max_rows,
-				 int max_columns,
+				 unsigned int col,
+				 unsigned int row,
+				 unsigned int width,
+				 unsigned int max_rows,
+				 unsigned int max_columns,
 				 float x,	float y,
 				 float w,	float h,
 				 void		(*position)(piece *p))
@@ -329,12 +346,12 @@ add_piece			(GdkPixbuf	*unscaled,
   p = matrix[row]->pieces + matrix[row]->n_pieces;
   matrix[row]->n_pieces++;
 
-  memset(p, 0, sizeof(*p));
+  CLEAR (*p);
 
   p->da = da;
 
-  gtk_signal_connect(GTK_OBJECT(p->da), "expose-event",
-		     GTK_SIGNAL_FUNC(expose), p);
+  g_signal_connect(G_OBJECT(p->da), "expose-event",
+		   G_CALLBACK(expose), p);
 
   p->x = x;
   p->y = y;
@@ -354,18 +371,21 @@ add_piece			(GdkPixbuf	*unscaled,
 }
 
 static void
-remove_piece			(int		row,
+remove_piece			(guint		row,
 				 int		p_index,
 				 gboolean	just_push)
 {
   piece *p = &(matrix[row]->pieces[p_index]);
 
   if (p->scaled)
-    gdk_pixbuf_unref(p->scaled);
-  gdk_pixbuf_unref(p->unscaled);
+    g_object_unref (G_OBJECT (p->scaled));
+  g_object_unref (G_OBJECT (p->unscaled));
 
-  gtk_signal_disconnect_by_func(GTK_OBJECT(p->da),
-				GTK_SIGNAL_FUNC(expose), p);
+  g_signal_handlers_disconnect_matched (G_OBJECT(p->da),
+					G_SIGNAL_MATCH_FUNC |
+					G_SIGNAL_MATCH_DATA,
+					0, 0, NULL,
+					G_CALLBACK (expose), p);
   
   if (!just_push)
     unget_window(p->da);
@@ -383,7 +403,7 @@ remove_piece			(int		row,
 }
 
 static void
-clear_row			(int		row,
+clear_row			(guint		row,
 				 gboolean	just_push)
 {
   while (matrix[row]->n_pieces)
@@ -393,16 +413,17 @@ clear_row			(int		row,
 void
 osd_clear			(void)
 {
-  int i;
+  guint i;
 
-  for (i=0; i<(MAX_ROWS-1); i++)
-    clear_row(i, FALSE);
+  for (i = 0; i < (MAX_ROWS - 1); ++i)
+    if (NULL != matrix[i])
+      clear_row (i, FALSE);
 
-  zmodel_changed(osd_model);
+  zmodel_changed (osd_model);
 }
 
 static void
-roll_up(int first_row, int last_row)
+roll_up(guint first_row, guint last_row)
 {
   gint i;
   float y = 0, h = 0;
@@ -419,7 +440,7 @@ roll_up(int first_row, int last_row)
       y_set = TRUE;
     }
 
-  clear_row(first_row, FALSE);
+  clear_row (first_row, FALSE);
   
   for (; first_row < last_row; first_row++)
     {
@@ -466,34 +487,41 @@ roll_up(int first_row, int last_row)
  * OSD sources.
  */
 
-#include "zvbi.h"
+#ifdef HAVE_LIBZVBI
+#include "src/zvbi.h"
 
 static vbi_page osd_page;
 extern int osd_pipe[2];
+static gint input_id = -1;
 
 static void
 ttx_position		(piece		*p)
 {
-  gint x = 0, y = 0, w = cw, h = ch;
+  gint x = cx, y = cy, w = cw, h = ch;
+  gint scol;
+  gint srow;
 
   /* Text area 40x25 is (64, 38) - (703, 537) in a 768x576 screen */
-  x += (64*w)/768;
-  y += (38*h)/576;
-  w -= (128*w)/768;
-  h -= (76*h)/576;
+  x += (64 * w) / 768;
+  y += (38 * h) / 576;
+  w -= (128 * w) / 768;
+  h -= (76 * h) / 576;
 
-  p->sw = p->w = ((p->column+p->width)*w)/p->max_columns
-    - (p->column*w)/p->max_columns;
-  p->h = ((p->row+1)*h)/p->max_rows-(p->row*h)/p->max_rows;
+  scol = (p->column * w) / p->max_columns;
+  srow = (p->row * h) / p->max_rows;
 
-  p->x = x + (p->column*w)/p->max_columns;
-  p->y = y + (p->row*h)/p->max_rows;
+  p->w = ((p->column + p->width) * w) / p->max_columns - scol;
+  p->h = ((p->row + 1) * h) / p->max_rows - srow;
+  p->sw = p->w;
+
+  p->x = x + scol;
+  p->y = y + srow;
 }
 
 static void
 cc_position		(piece		*p)
 {
-  gint x = 0, y = 0, w = cw, h = ch;
+  gint x = cx, y = cy, w = cw, h = ch;
   gint width0 /* min width of each char */,
     extra /* pixels remaining for completing total width */,
     total /* total width of the line */;
@@ -540,7 +568,7 @@ cc_position		(piece		*p)
 }
 
 static void
-add_piece_vbi		(int col, int row, int width)
+add_piece_vbi		(guint col, guint row, guint width)
 {
   GdkPixbuf *buf;
   GtkWidget *da;
@@ -550,27 +578,30 @@ add_piece_vbi		(int col, int row, int width)
   if (osd_page.columns < 40) /* naive cc test */
     {
       buf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
-			   16 * width, 26);
+			   (int)(16 * width), 26);
       vbi_draw_cc_page_region(&osd_page,
 			      VBI_PIXFMT_RGBA32_LE,
 			      (uint32_t *) gdk_pixbuf_get_pixels(buf),
 			      gdk_pixbuf_get_rowstride(buf),
-			      col, row, width, 1 /* height */);
+			      (int) col, (int) row, (int) width,
+			      1 /* height */);
     }
   else
     {
       buf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
-			   12 * width, 10);
+			   (gint)(12 * width), 10);
       vbi_draw_vt_page_region(&osd_page,
 			      VBI_PIXFMT_RGBA32_LE,
 			      (uint32_t *) gdk_pixbuf_get_pixels(buf),
 			      gdk_pixbuf_get_rowstride(buf),
-			      col, row, width, 1 /* height */,
+			      (int) col, (int) row, (int) width,
+			      1 /* height */,
 			      1 /* reveal */, 1 /* flash_on */);
     }
 
-  add_piece(buf, da, col, row, width, osd_page.rows,
-	    osd_page.columns, 0, 0, 0, 0,
+  add_piece(buf, da, col, row, width,
+	    (guint) osd_page.rows,
+	    (guint) osd_page.columns, 0.0, 0.0, 0.0, 0.0,
 	    (osd_page.columns < 40) ? cc_position : ttx_position);
 }
 
@@ -578,16 +609,17 @@ static void
 render_page		(void)
 {
   vbi_char *ac_row;
-  int row, i, j;
+  guint row;
+  guint i, j;
   gboolean dirty = FALSE;
 
   row = osd_page.dirty.y0;
   ac_row = osd_page.text + row * osd_page.columns;
 
-  for (; row <= osd_page.dirty.y1; ac_row += osd_page.columns, row++)
+  for (; row <= (guint) osd_page.dirty.y1; ac_row += osd_page.columns, row++)
     {
       clear_row(row, TRUE);
-      for (i = j = 0; i < osd_page.columns; i++)
+      for (i = j = 0; i < (guint) osd_page.columns; i++)
         {
 	  if (ac_row[i].opacity != VBI_TRANSPARENT_SPACE)
 	    j++;
@@ -613,14 +645,12 @@ render_page		(void)
 }
 
 static void
-osd_event		(gpointer	   data,
-			 gint              source, 
-			 GdkInputCondition condition)
+osd_event		(gpointer	   data _unused_,
+			 gint              source _unused_, 
+			 GdkInputCondition condition _unused_)
 {
   vbi_decoder *vbi = zvbi_get_object();
   char dummy[16];
-  extern vbi_pgno zvbi_page;
-  extern vbi_subno zvbi_subpage;
 
   if (!vbi)
     return;
@@ -631,32 +661,23 @@ osd_event		(gpointer	   data,
   if (!zconf_get_boolean(NULL, "/zapping/internal/callbacks/closed_caption"))
     return;
 
-  if (zvbi_page <= 8)
-    {
-      if (!vbi_fetch_cc_page(vbi, &osd_page, zvbi_page, TRUE))
-        return; /* trouble in outer space */
-    }
-  else
-    {
-      if (!vbi_fetch_vt_page(vbi, &osd_page, zvbi_page, zvbi_subpage,
-			     zvbi_teletext_level(), 25 /* rows */, TRUE /* nav */))
-        return;
-    }
+ switch (zvbi_caption_pgno)
+   {
+   case 1 ... 8:
+     if (!vbi_fetch_cc_page(vbi, &osd_page, zvbi_caption_pgno, TRUE))
+       return; /* trouble in outer space */
+     break;
 
-#if 0
-  {
-    int row, col;
+   case 0x100 ... 0x899:
+     if (!vbi_fetch_vt_page(vbi, &osd_page,
+			    zvbi_caption_pgno, VBI_ANY_SUBNO,
+			    VBI_WST_LEVEL_1p5, 25 /* rows */, TRUE /* nav */))
+       return;
+     break;
 
-    fprintf (stderr, "osd_event page=%d dirty.y0=%d .y1=%d .roll=%d\n",
-      osd_page.pgno, osd_page.dirty.y0, osd_page.dirty.y1, osd_page.dirty.roll);
-    for (row = osd_page.dirty.y0; row <= osd_page.dirty.y1; row++)
-      {
-        for (col = 0; col < osd_page.columns; col++)
-          fputc (osd_page.text[row * osd_page.columns + col].glyph & 0x7F, stderr);
-        fputc ('\n', stderr);
-      }
-  }
-#endif
+   default:
+     return;
+   }
 
   if (osd_page.dirty.y0 > osd_page.dirty.y1)
     return; /* not dirty (caption only) */
@@ -669,7 +690,8 @@ osd_event		(gpointer	   data,
 
   if (osd_page.dirty.roll == -1)
     {
-      roll_up(osd_page.dirty.y0, osd_page.dirty.y1);
+      roll_up ((guint) osd_page.dirty.y0,
+	       (guint) osd_page.dirty.y1);
       return;
     }
 
@@ -678,101 +700,28 @@ osd_event		(gpointer	   data,
 
   render_page();
 }
+#endif /* HAVE_LIBZVBI */
 
-/* OSD text provided by the app */
-#include <parser.h> /* libxml */
+#include <libxml/parser.h>
 
-#define ATTR_STACK	128
-typedef struct {
-  unsigned int		italic;
-  unsigned int		bold;
-  unsigned int		flash;
-  unsigned int		underline;
-  unsigned int		fg[ATTR_STACK];
-  unsigned int		fg_sp;
-  unsigned int		opacity[ATTR_STACK];
-  unsigned int		opacity_sp;
+static guint osd_clear_timeout_id = NO_SOURCE_ID;
+static void (* osd_clear_timeout_cb)(gboolean);
 
-  /* For drawing in OSD mode */
-  unsigned int		x, baseline; /* paint pos */
-  unsigned int		ascent, descent; /* maximum used */
-  GdkPixmap		*canvas;
-  GdkGC			*gc;
+#define OSD_ROW (MAX_ROWS - 1)
 
-  /* Other OSD fields */
-  GdkColormap		*cmap;
-  GtkWidget		*patch;
-  unsigned int		start_x;
-  int			lbearing;
-
-  /* Status bar mode */
-  gchar			*dest_buf;
-
-} sax_context;
-
-#define sec(label, entry, entry_sp) \
-else if (!strcasecmp(name, #label) && ctx->entry_sp < (ATTR_STACK-1)) \
-  ctx->entry[++ctx->entry_sp] = VBI_##label
-
-static void
-my_startElement (void *ptr,
-		 const xmlChar *name, const xmlChar **atts)
+static gint
+osd_clear_timeout	(void		*ignored _unused_)
 {
-  sax_context *ctx = (sax_context *) ptr;
+  clear_row(OSD_ROW, FALSE);
 
-  if (!strcasecmp(name, "i"))
-    ctx->italic ++;
-  else if (!strcasecmp(name, "b"))
-    ctx->bold++;
-  else if (!strcasecmp(name, "u"))
-    ctx->underline++;
-  else if (!strcasecmp(name, "f"))
-    ctx->flash++;
+  osd_clear_timeout_id = NO_SOURCE_ID;
 
-  sec(TRANSPARENT_SPACE, opacity, opacity_sp);
-  sec(TRANSPARENT_FULL, opacity, opacity_sp);
-  sec(SEMI_TRANSPARENT, opacity, opacity_sp);
-  sec(OPAQUE, opacity, opacity_sp);
+  zmodel_changed(osd_model);
 
-  sec(BLACK, fg, fg_sp);
-  sec(RED, fg, fg_sp);
-  sec(GREEN, fg, fg_sp);
-  sec(YELLOW, fg, fg_sp);
-  sec(BLUE, fg, fg_sp);
-  sec(MAGENTA, fg, fg_sp);
-  sec(CYAN, fg, fg_sp);
-  sec(WHITE, fg, fg_sp);
-}
+  if (osd_clear_timeout_cb)
+    osd_clear_timeout_cb(TRUE);
 
-#define eec(label, entry_sp) \
-else if (!strcasecmp(name, #label) && ctx->entry_sp) \
-ctx->entry_sp--
-
-static void my_endElement (void *ptr,
-			   const xmlChar *name)
-{
-  sax_context *ctx = (sax_context *) ptr;
-
-  if (!strcasecmp(name, "i") && ctx->italic)
-    ctx->italic--;
-
-  eec(b, bold);
-  eec(u, underline);
-  eec(f, flash);
-
-  eec(TRANSPARENT_SPACE, opacity_sp);
-  eec(TRANSPARENT_FULL, opacity_sp);
-  eec(SEMI_TRANSPARENT, opacity_sp);
-  eec(OPAQUE, opacity_sp);
-
-  eec(BLACK, fg_sp);
-  eec(RED, fg_sp);
-  eec(GREEN, fg_sp);
-  eec(YELLOW, fg_sp);
-  eec(BLUE, fg_sp);
-  eec(MAGENTA, fg_sp);
-  eec(CYAN, fg_sp);
-  eec(WHITE, fg_sp);
+  return FALSE;
 }
 
 /**
@@ -805,301 +754,172 @@ unref_color		(GdkColor *color, GdkColormap *cmap)
   g_free(color);
 }
 
-/* Takes foreground and produces a suitable color */
-static GdkColor *
-create_color_by_id	(int color_no,
-			 GdkColormap *cmap)
+static gchar *
+remove_markup			(const gchar *		s)
 {
-  GdkColor c, *returned_color;
-  const char *colors[] =
-  {
-    "black", "red", "green", "yellow", "blue", "magenta", "cyan",
-    "white"
-  };
-  const char *color;
-  
-  if (color_no < 0 || color_no >= acount(colors))
-    color = "white";
-  else
-    color = colors[color_no];
-  
-  if (gdk_color_parse(color, &c))
-    returned_color = create_color(c.red / 65535.0,
-				  c.green / 65535.0,
-				  c.blue / 65535.0, cmap);
-  else
-    returned_color = create_color(1.0, 1.0, 1.0, cmap);
-  
-  return returned_color;
-}
+  GError *error = NULL;
+  gchar *d;
 
-/**
- * Creates the given font from the given template, returns NULL on
- * error or a GdkFont* to be freed with unref_font.
- */
-static GdkFont *
-create_font		(const		gchar *_xlfd,
-			 gboolean	bold,
-			 gboolean	italic)
-{
-  const gchar *xlfd = _xlfd;
-  gchar *buf = g_malloc0(strlen(xlfd)+30), *p=buf;
-  GdkFont *result;
+  pango_parse_markup (s, -1, 0, NULL, &d, NULL, &error);
 
-  if (!*xlfd)
-    return NULL;
-
-  /* Skip foundry and family */
-  *(p++) = *(xlfd++);
-  while (*xlfd && *xlfd != '-')
-    *(p++) = *(xlfd++);
-  *(p++) = '-';
-  xlfd++;
-
-  while (*xlfd && *xlfd != '-')
-    *(p++) = *(xlfd++);
-  *(p++) = '-';
-  xlfd++;
-
-  /* Weight and Slant */
-  p += sprintf(p, "%s-%s", bold? "bold" : "medium", italic ? "i" : "r");
-
-  while (*xlfd && *xlfd != '-')
-    xlfd++;
-  xlfd++;
-  while (*xlfd && *xlfd != '-')
-    xlfd++;
-
-  /* the rest of the xlfd */
-  strcpy(p, xlfd);
-
-  result = gdk_font_load(buf);
-
-  if (!result)
-    result = gdk_font_load(_xlfd);
-
-  g_free(buf);
-
-  if (result)
-    gdk_font_ref(result);
-
-  return result;
-}
-
-/**
- * Call this when you don't need the font any longer
- */
-static void
-unref_font		(GdkFont *font)
-{
-  gdk_font_unref(font);
-}
-
-static void my_characters (void *ptr,
-			   const xmlChar *ch, int n)
-{
-  sax_context *ctx = ptr;
-  gboolean underline, bold, italic /*, flash : ignored */;
-  /* int opacity; :ignored */
-  int foreground;
-  GdkColor *fg;
-  GdkColor *bg;
-  GdkFont *font;
-  gchar *buf;
-  gint i, j;
-  gint lbearing, rbearing, width, ascent, descent;
-
-  buf = g_malloc0(n*sizeof(char) + 1);
-
-  for (i=j=0; i<n; i++)
-    if (ch[i] != '\n' && ch[i] != '\r')
-      buf[j++] = ch[i];
-
-  buf[j] = 0;
-
-  if (!j)
-    goto done;
-
-  underline = ctx->underline > 0;
-  bold = ctx->bold > 0;
-  italic = ctx->italic > 0;
-  /* flash = ctx->flash > 0; */
-  /* c.opacity = ctx->opacity_sp ? ctx->opacity[ctx->opacity_sp] :
-     OPAQUE; */
-  foreground = ctx->fg_sp ? ctx->fg[ctx->fg_sp] : -1;
-
-  switch (zcg_int(NULL, "osd_type"))
+  if (!d || error)
     {
-    case 1:
-      {
-	gchar *buf2 = g_strconcat(ctx->dest_buf, buf, NULL);
-	g_free(ctx->dest_buf);
-	ctx->dest_buf = buf2;
-      }
-      goto done;
-    case 2:
-      printf("%s", buf);
-      goto done;
-    default:
-      break;
+      g_assert (!d);
+
+      if (error)
+	{
+	  if (OSD_TEST)
+	    fprintf (stderr, "%s:%u: Bad markup in \"%s\": %s\n",
+		     __FILE__, __LINE__, s, error->message);
+
+	  g_error_free (error);
+	  error = NULL;
+	}
+
+      return NULL;
     }
-  
-  bg = create_color(zcg_float(NULL, "bg_r"), zcg_float(NULL, "bg_g"),
-		    zcg_float(NULL, "bg_b"), ctx->cmap);
 
-  if (foreground >= 0)
-    fg = create_color_by_id(foreground, ctx->cmap);
-  else
-    fg = create_color(zcg_float(NULL, "fg_r"), zcg_float(NULL, "fg_g"),
-		      zcg_float(NULL, "fg_b"), ctx->cmap);
-
-  font = create_font(zcg_char(NULL, "font"), bold, italic);
-
-  g_assert(font != NULL);
-
-  /* Draw and increment the cursor pos */
-  gdk_gc_set_background(ctx->gc, bg);
-  gdk_gc_set_foreground(ctx->gc, fg);
-
-  gdk_draw_string(ctx->canvas, font, ctx->gc,
-		  ctx->x, ctx->baseline, buf);
-
-  /* update x, ascent, descent */
-  gdk_string_extents(font, buf, &lbearing, &rbearing, &width, &ascent,
-		     &descent);
-
-  if (underline)
-    gdk_draw_line(ctx->canvas, ctx->gc, ctx->x, ctx->baseline+1,
-		  ctx->x+width-1, ctx->baseline+1);
-
-  ctx->x += width;
-  ctx->ascent = MAX(ctx->ascent, ascent);
-  ctx->descent = MAX(ctx->descent, descent);
-
-  if (ctx->lbearing < 0)
-    ctx->lbearing = lbearing;
-
-  unref_font(font);
-  unref_color(fg, ctx->cmap);
-  unref_color(bg, ctx->cmap);
-
- done:
-  g_free(buf);
+  return d;
 }
 
-static gboolean
-prepare_context		(sax_context	*ctx,
-			 const gchar	*buf)
+static void
+render_console			(const gchar *		s,
+				 gboolean		markup)
 {
-  GdkFont *font;
-  gchar *fname = zcg_char(NULL, "font");
-  gint lbearing, rbearing, width, ascent, descent;
-  gint w, h;
+  GError *error = NULL;
+  gchar *plain_buf;
+  gchar *locale_buf;
+  gsize length;
+  guint i;
+
+  if (markup)
+    {
+      plain_buf = remove_markup (s);
+
+      if (!plain_buf)
+	return;
+    }
+  else
+    {
+      plain_buf = g_strdup (s);
+    }
+
+  for (i = 0; 0 != plain_buf[i]; ++i)
+    if ('\n' == plain_buf[i] ||
+	'\r' == plain_buf[i])
+      plain_buf[i] = ' ';
+
+  locale_buf = g_locale_from_utf8 (plain_buf, (gint) i, NULL, &length, &error);
+
+  if (!locale_buf || error)
+    {
+      g_assert (!locale_buf);
+
+      if (error)
+	{
+	  if (OSD_TEST)
+	    fprintf (stderr, "%s:%u: Bad UTF-8 in \"%s\": %s\n",
+		     __FILE__, __LINE__, plain_buf, error->message);
+
+	  g_error_free (error);
+	  error = NULL;
+	}
+    }
+  else
+    {
+      if (length > 0)
+	{
+	  fputs (locale_buf, stdout);
+	  
+	  if (locale_buf[length - 1] != '\n')
+	    fputc ('\n', stdout);
+	}
+
+      g_free (locale_buf);
+    }
+
+  g_free (plain_buf);
+}
+
+/* Render in OSD mode. Put markup to TRUE if the text contains pango
+   markup to be interpreted. */
+static void
+osd_render_osd		(void (*timeout_cb)(gboolean),
+			 const gchar *src, gboolean markup)
+{
+  GdkDrawable *canvas;
+  GtkWidget *patch = pop_window ();
+  PangoContext *context = gtk_widget_get_pango_context (patch);
+  PangoLayout *layout = pango_layout_new (context);
+  PangoFontDescription *pfd = NULL;
+  GdkGC *gc = gdk_gc_new (patch->window);
+  GdkColormap *cmap = gdk_drawable_get_colormap (patch->window);
+  const gchar *fname = zcg_char (NULL, "font");
+  PangoRectangle logical, ink;
   GdkColor *bg;
+  gint w, h;
+  gfloat rx, ry, rw, rh;
+  gchar *buf = g_strdup_printf ("<span foreground=\"#%02x%02x%02x\">%s</span>",
+				(int)(zcg_float(NULL, "fg_r")*255),
+				(int)(zcg_float(NULL, "fg_g")*255),
+				(int)(zcg_float(NULL, "fg_b")*255), src);
 
   /* First check that the selected font is valid */
   if (!fname || !*fname ||
-      !(font = create_font(fname, TRUE, TRUE)))
+      !(pfd = pango_font_description_from_string (fname)))
     {
       if (!fname || !*fname)
-	ShowBox(_("Please configure a font for OSD in\n"
-		  "Properties/General/OSD"),
-		GNOME_MESSAGE_BOX_ERROR);
+	ShowBox(_("Please choose a font for OSD in preferences."),
+		GTK_MESSAGE_ERROR);
       else
-	ShowBox(_("The configured font <%s> cannot be loaded, please"
-		  " select another one in\n"
-		  "Properties/General/OSD"),
-		GNOME_MESSAGE_BOX_ERROR, fname);
-      
-      return FALSE;
+	ShowBox(_("The configured font \"%s\" cannot be loaded, please"
+		  " select another one in preferences\n"),
+		GTK_MESSAGE_ERROR, fname);
+
+      /* Some common fonts */
+      pfd = pango_font_description_from_string ("Arial 36");
+      if (!pfd)
+	pfd = pango_font_description_from_string ("Sans 36");
+      if (!pfd)
+	{
+	  ShowBox(_("No font could be loaded, please make sure\n"
+		    "your system is properly configured."),
+		  GTK_MESSAGE_ERROR);
+	  unget_window (patch);
+	  g_object_unref (G_OBJECT (layout));
+	  g_object_unref (G_OBJECT (gc));
+	  g_free (buf);
+	  return;
+	}
     }
 
-  ctx->patch = pop_window();
+  pango_layout_set_font_description (layout, pfd);
 
-  ctx->gc = gdk_gc_new(ctx->patch->window);
-  g_assert(ctx->gc != NULL);
-  ctx->cmap = gdk_window_get_colormap(ctx->patch->window);
+  if (markup)
+    pango_layout_set_markup (layout, buf, -1);
+  else
+    pango_layout_set_text (layout, buf, -1);
 
-  gdk_string_extents(font, buf, &lbearing, &rbearing, &width, &ascent,
-		     &descent);
+  /* Get the text extents, compute the geometry and build the canvas */
+  pango_layout_get_pixel_extents (layout, &ink, &logical);
 
-  ctx->lbearing = -1;
+  w = logical.width;
+  h = logical.height;
 
-  /*
-    We create a pixmap much bigger than strictly neeeded, we'll only
-    convert the appropiate rectangle in process_contents, where we
-    know for sure the dimensions. We save some complexity this way at
-    the cost of performance (not noticeable anyway).
-  */
-  w = width*2.5;
-  ctx->start_x = ctx->x = width/4;
-  h = (ascent+descent)*4;
-  ctx->baseline = ascent*2;
+  canvas = gdk_pixmap_new (patch->window, w, h, -1);
 
-  ctx->canvas = gdk_pixmap_new(ctx->patch->window, w, h, -1);
-  g_assert(ctx->canvas);
+  /* Draw the canvas contents */
+  bg = create_color(zcg_float(NULL, "bg_r"),
+		    zcg_float(NULL, "bg_g"),
+		    zcg_float(NULL, "bg_b"), cmap);
 
-  bg = create_color(zcg_float(NULL, "bg_r"), zcg_float(NULL, "bg_g"),
-		    zcg_float(NULL, "bg_b"), ctx->cmap);
-  gdk_gc_set_foreground(ctx->gc, bg);
-  gdk_draw_rectangle(ctx->canvas, ctx->gc, TRUE, 0, 0, w, h);
-  unref_color(bg, ctx->cmap);
+  gdk_gc_set_foreground(gc, bg);
+  gdk_draw_rectangle(canvas, gc, TRUE, 0, 0, w, h);
+  unref_color(bg, cmap);
 
-  unref_font(font);
-  return TRUE;
-}
-
-static gint osd_clear_timeout_id = -1;
-static void (* osd_clear_timeout_cb)(gboolean);
-
-#define OSD_ROW (MAX_ROWS - 1)
-
-static gint
-osd_clear_timeout	(void		*ignored)
-{
-  clear_row(OSD_ROW, FALSE);
-
-  osd_clear_timeout_id = -1;
-
-  zmodel_changed(osd_model);
-
-  if (osd_clear_timeout_cb)
-    osd_clear_timeout_cb(TRUE);
-
-  return FALSE;
-}
-
-/* Finish the patch by creating the gdk pixbuf, etc */
-static void
-process_context		(sax_context	*ctx,
-			 void (* timeout_cb)(gboolean))
-{
-  gint w, h, x, y, hmargin, vmargin;
-  float rx, ry, rw, rh;
-
-  w = (ctx->lbearing + (ctx->x - ctx->start_x));
-  h = (ctx->ascent + ctx->descent);
-
-  hmargin = MIN(40, w*0.6);
-  vmargin = MIN(40, h*0.6);
-
-  x = ctx->start_x - (ctx->lbearing + hmargin/2);
-  y = ctx->baseline - (ctx->ascent + vmargin/2);
-
-  w += hmargin;
-  h += vmargin;
-
-  if (x<0)
-    {
-      w+=x;
-      x=0;
-    }
-  if (y<0)
-    {
-      h+=y;
-      h=0;
-    }
+  gdk_draw_layout (canvas, gc, 0, 0, layout);
 
+  /* Compute the resulting patch geometry */
   rh = 0.1;
   rw = (rh*w)/h;
 
@@ -1112,214 +932,178 @@ process_context		(sax_context	*ctx,
   rx = 1 - rw;
   ry = 1 - rh;
 
-  clear_row(OSD_ROW, FALSE);
-
+  /* Create the patch */
   add_piece(gdk_pixbuf_get_from_drawable
-	    (NULL, ctx->canvas, ctx->cmap, x, y, 0, 0, w, h),
-	    ctx->patch,
-	    0, OSD_ROW, 0, 0, 0, rx, ry, rw, rh, NULL);
+	    (NULL, canvas, cmap, 0, 0, 0, 0, w, h),
+	    patch, 0, OSD_ROW, 0, 0, 0, rx, ry, rw, rh, NULL);
 
   zmodel_changed(osd_model);
 
-  if (osd_clear_timeout_id > -1)
+  /* Schedule the destruction of the patch */
+  if (osd_clear_timeout_id > 0)
     {
       if (osd_clear_timeout_cb)
 	osd_clear_timeout_cb(FALSE);
 
-      gtk_timeout_remove(osd_clear_timeout_id);
+      g_source_remove (osd_clear_timeout_id);
     }
 
-  gdk_pixmap_unref(ctx->canvas);
-
   osd_clear_timeout_id =
-    gtk_timeout_add(zcg_float(NULL, "timeout")*1000,
-		    osd_clear_timeout, NULL);
+    g_timeout_add ((guint)(zcg_float (NULL, "timeout") * 1000),
+		   (GSourceFunc) osd_clear_timeout, NULL);
 
   osd_clear_timeout_cb = timeout_cb;
+
+  /* Cleanup */
+  pango_font_description_free (pfd);
+  g_object_unref (G_OBJECT (layout));
+  g_object_unref (G_OBJECT (canvas));
+  g_object_unref (G_OBJECT (gc));
+  g_free (buf);
 }
 
 /* If given, timeout_cb(TRUE) is called when osd timed out,
    timeout_cb(FALSE) when error, replaced.
 */
 void
-osd_render_sgml		(void (*timeout_cb)(gboolean),
-			 const char *string, ...)
+osd_render_markup		(osd_timeout_fn *	timeout_cb,
+				 osd_type		type,
+				 const char *		buf)
 {
-  va_list args;
-  sax_context ctx;
-  xmlSAXHandler handler;
-  gchar *buf, *buf2;
-  int error;
-  
-  if (!string || !string[0])
+  if (!buf || !buf[0])
     goto failed;
 
-  va_start(args, string);
-  buf = g_strdup_vprintf(string, args);
-  va_end(args);
+  if (OSD_TYPE_CONFIG == type)
+    type = zcg_int (NULL, "osd_type");
 
-  if (!buf)
-    goto failed;
-
-  if (!buf[0])
+  /* The different ways of drawing */
+  switch (type)
     {
-      g_free(buf);
-      goto failed;
-    }
-
-  memset(&ctx, 0, sizeof(ctx));
-
-  memset(&handler, 0, sizeof(handler));
-
-  handler.startElement = my_startElement;
-  handler.endElement = my_endElement;
-  handler.characters = my_characters;
-
-  buf2 = g_strdup_printf("<?xml version=\"1.0\" encoding=\"%s\"?>"
-			 "<doc><text>%s</text></doc>    ",
-			 get_locale_charset(), buf);
-
-  /* Prepare for drawing */
-  switch (zcg_int(NULL, "osd_type"))
-    {
-    case 0:
-      if (!prepare_context(&ctx, buf))
-	{
-	  g_free(buf2);
-	  g_free(buf);
-	  goto failed;
-	}
+    case 0: /* OSD */
+      clear_row (OSD_ROW, TRUE);
+      osd_render_osd (timeout_cb, buf, TRUE);
       break;
-    case 1:
-      ctx.dest_buf = g_strdup("");
-      break;
-    case 2:
-      printf("OSD: ");
+
+    case 1: /* Statusbar */
+      {
+	gchar *plain_buf;
+
+	/* Color looks funny in status bar. */
+	if (!(plain_buf = remove_markup (buf)))
+	  break;
+
+	z_status_print (plain_buf, /* markup */ FALSE,
+			(guint)(zcg_float (NULL, "timeout") * 1000),
+			/* hide */ FALSE);
+
+	g_free (plain_buf);
+
+	break;
+      }
+
+    case 2: /* Console */
+      render_console (buf, /* markup */ TRUE);
       break;
+
+    case 3:
+      break; /* Ignore */
+
     default:
-      g_assert_not_reached();
+      g_assert_not_reached ();
       break;
     }
 
-  error = xmlSAXUserParseMemory(&handler, &ctx, buf2, strlen(buf2));
-
-  if (error != 0)
-    {
-      g_warning("Couldn't parse XML string '%s' in '%s', error code %d",
-		buf, buf2, error);
-      g_free(buf2);
-      g_free(buf);
-      goto failed;
-    }
-
-  /* Create the patch with the pixbuf contents */
-  switch (zcg_int(NULL, "osd_type"))
-    {
-    case 0:
-      process_context(&ctx, timeout_cb);
-      break;
-    case 1:
-      z_status_print(ctx.dest_buf);
-      g_free(ctx.dest_buf);
-      break;
-    case 2:
-      printf("\n");
-      break;
-    default:
-      g_assert_not_reached();
-      break;
-    }
-
-  g_free(buf2);
-  g_free(buf);
-
   return;
 
  failed:
-
   if (timeout_cb)
-    timeout_cb(FALSE);
+    timeout_cb (FALSE);
+}
 
-  return;
+/* If given, timeout_cb(TRUE) is called when osd timed out,
+   timeout_cb(FALSE) when error, replaced.
+*/
+void
+osd_render_markup_printf	(osd_timeout_fn *	timeout_cb,
+				 const char *		template,
+				 ...)
+{
+  gchar *buf;
+  va_list args;
+
+  buf = NULL;
+
+  if (!template || !template[0])
+    return;
+
+  va_start (args, template);
+
+  buf = g_strdup_vprintf (template, args);
+
+  va_end (args);
+
+  osd_render_markup (timeout_cb, OSD_TYPE_CONFIG, buf);
+
+  g_free (buf);
 }
 
 void
-osd_render		(void (*timeout_cb)(gboolean),
-			 const char *string, ...)
+osd_render			(osd_timeout_fn *	timeout_cb,
+				 const char *		template,
+				 ...)
 {
-  va_list args;
-  sax_context ctx;
   gchar *buf;
+  va_list args;
 
-  if (!string || !string[0])
+  buf = NULL;
+
+  if (!template || !template[0])
     goto failed;
 
-  va_start(args, string);
-  buf = g_strdup_vprintf(string, args);
-  va_end(args);
+  va_start (args, template);
 
-  if (!buf)
+  buf = g_strdup_vprintf (template, args);
+
+  va_end (args);
+
+  if (!buf || !buf[0])
     goto failed;
 
-  if (!*buf)
+  /* The different ways of drawing */
+  switch (zcg_int (NULL, "osd_type"))
     {
-      g_free(buf);
-      goto failed;
-    }
+    case 0: /* OSD */ 
+      clear_row (OSD_ROW, TRUE);
+      osd_render_osd (timeout_cb, buf, FALSE);
+      break;
 
-  memset(&ctx, 0, sizeof(ctx));
+    case 1: /* Statusbar */
+      z_status_print (buf, /* markup */ FALSE,
+		      (guint)(zcg_float (NULL, "timeout") * 1000),
+		      /* hide */ FALSE);
+      break;
 
-  /* Prepare for drawing */
-  switch (zcg_int(NULL, "osd_type"))
-    {
-    case 0:
-      if (!prepare_context(&ctx, buf))
-	{
-	  g_free(buf);
-	  goto failed;
-	}
-      break;
-    case 1:
-      ctx.dest_buf = g_strdup("");
-      break;
-    case 2:
-      printf("OSD: ");
+    case 2: /* Console */
+      render_console (buf, /* markup */ FALSE);
       break;
+
+    case 3:
+      break; /* Ignore */
+
     default:
-      g_assert_not_reached();
+      g_assert_not_reached ();
       break;
     }
 
-  my_characters(&ctx, buf, strlen(buf));
-
-  /* Create the patch with the pixbuf contents */
-  switch (zcg_int(NULL, "osd_type"))
-    {
-    case 0:
-      process_context(&ctx, timeout_cb);
-      break;
-    case 1:
-      z_status_print(ctx.dest_buf);
-      g_free(ctx.dest_buf);
-      break;
-    case 2:
-      printf("\n");
-      break;
-    default:
-      g_assert_not_reached();
-      break;
-    }
-
-  g_free(buf);
+  g_free (buf);
 
   return;
 
  failed:
+  g_free (buf);
 
   if (timeout_cb)
-    timeout_cb(FALSE);
-
-  return;
+    timeout_cb (FALSE);
 }
 
 /**
@@ -1328,9 +1112,9 @@ osd_render		(void (*timeout_cb)(gboolean),
 gboolean coords_mode = FALSE;
 
 static
-void on_osd_screen_size_allocate	(GtkWidget	*widget,
+void on_osd_screen_size_allocate	(GtkWidget	*widget _unused_,
 					 GtkAllocation	*allocation,
-					 gpointer	ignored)
+					 gpointer	ignored _unused_)
 {
   if (cw == allocation->width &&
       ch == allocation->height)
@@ -1346,18 +1130,22 @@ static void
 set_window(GtkWidget *dest_window, gboolean _coords_mode)
 {
   if (osd_window && !coords_mode)
-    gtk_signal_disconnect_by_func(GTK_OBJECT(osd_window),
-				  GTK_SIGNAL_FUNC(on_osd_screen_size_allocate),
-				  NULL);
+    g_signal_handlers_disconnect_matched (G_OBJECT(osd_window),
+					  G_SIGNAL_MATCH_FUNC |
+					  G_SIGNAL_MATCH_DATA,
+					  0, 0, NULL,
+					  G_CALLBACK
+					  (on_osd_screen_size_allocate),
+					  NULL);
 
   osd_window = dest_window;
 
   coords_mode = _coords_mode;
 
   if (!coords_mode)
-    gtk_signal_connect(GTK_OBJECT(dest_window), "size-allocate",
-		       GTK_SIGNAL_FUNC(on_osd_screen_size_allocate),
-		       NULL);
+    g_signal_connect(G_OBJECT(dest_window), "size-allocate",
+		     G_CALLBACK(on_osd_screen_size_allocate),
+		     NULL);
 
   geometry_update();
 }
@@ -1368,7 +1156,7 @@ osd_set_window(GtkWidget *dest_window)
   gtk_widget_realize(dest_window);
 
   cx = cy = 0;
-  gdk_window_get_size(dest_window->window, &cw, &ch);
+  gdk_drawable_get_size(dest_window->window, &cw, &ch);
   
   set_window(dest_window, FALSE);
 }
@@ -1392,18 +1180,185 @@ osd_unset_window(void)
     return;
 
   if (!coords_mode)
-    gtk_signal_disconnect_by_func(GTK_OBJECT(osd_window),
-				  GTK_SIGNAL_FUNC(on_osd_screen_size_allocate),
-				  NULL);
+    g_signal_handlers_disconnect_matched (G_OBJECT(osd_window),
+					  G_SIGNAL_MATCH_FUNC |
+					  G_SIGNAL_MATCH_DATA,
+					  0, 0, NULL,
+					  G_CALLBACK (on_osd_screen_size_allocate),
+					  NULL);
 
   osd_window = NULL;
   geometry_update(); /* Reparent to the orphanarium */
 }
 
+/* Python wrappers for the OSD renderer */
+static PyObject* py_osd_render (PyObject *self _unused_, PyObject *args)
+{
+  char *string;
+  int ok = ParseTuple (args, "s", &string);
+
+  if (!ok)
+    g_error ("zapping.osd_render(s)");
+
+  osd_render (NULL, "%s", string);
+
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+static PyObject* py_osd_render_markup (PyObject *self _unused_, PyObject *args)
+{
+  char *string;
+  int ok = ParseTuple (args, "s", &string);
+
+  if (!ok)
+    g_error ("zapping.osd_render_markup(s)");
+
+  osd_render_markup (NULL, OSD_TYPE_CONFIG, string);
+
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+static void
+on_osd_type_changed	(GtkWidget	*widget,
+			 GtkWidget	*page _unused_)
+{
+  GtkWidget *w;
+
+  gboolean sensitive1 = FALSE;
+  gboolean sensitive2 = FALSE;
+
+  widget = lookup_widget(widget, "optionmenu22");
+
+  switch (z_option_menu_get_active (widget))
+    {
+    case 0:
+      sensitive1 = TRUE;
+    case 1:
+      sensitive2 = TRUE;
+      break;
+    default:
+      break;
+    }
+
+  /* XXX Ugh. */
+
+  w = lookup_widget (widget, "general-osd-font-label");
+  gtk_widget_set_sensitive (w, sensitive1);
+  w = lookup_widget (widget, "general-osd-font-selector");
+  gtk_widget_set_sensitive (w, sensitive1);
+  w = lookup_widget (widget, "general-osd-foreground-label");
+  gtk_widget_set_sensitive (w, sensitive1);
+  w = lookup_widget (widget, "general-osd-foreground-selector");
+  gtk_widget_set_sensitive (w, sensitive1);
+  w = lookup_widget (widget, "general-osd-background-label");
+  gtk_widget_set_sensitive (w, sensitive1);
+  w = lookup_widget (widget, "general-osd-background-selector");
+  gtk_widget_set_sensitive (w, sensitive1);
+
+  w = lookup_widget (widget, "general-osd-timeout-label");
+  gtk_widget_set_sensitive (w, sensitive2);
+  w = lookup_widget (widget, "general-osd-timeout-selector");
+  gtk_widget_set_sensitive (w, sensitive2);
+}
+
+/* OSD properties */
+static void
+osd_setup		(GtkWidget	*page)
+{
+  GtkWidget *widget;
+
+  /* OSD type */
+  widget = lookup_widget(page, "optionmenu22");
+  gtk_option_menu_set_history(GTK_OPTION_MENU(widget),
+			      (guint) zcg_int(NULL, "osd_type"));
+  on_osd_type_changed (page, page);
+
+  g_signal_connect(G_OBJECT (widget), "changed",
+		   G_CALLBACK(on_osd_type_changed),
+		   page);
+
+  /* OSD font */
+  widget = lookup_widget(page, "general-osd-font-selector");
+  if (zcg_char(NULL, "font"))
+    gnome_font_picker_set_font_name(GNOME_FONT_PICKER(widget),
+				    zcg_char(NULL, "font"));
+
+  /* OSD foreground color */
+  widget = lookup_widget(page, "general-osd-foreground-selector");
+  gnome_color_picker_set_d(GNOME_COLOR_PICKER(widget),
+		   zcg_float(NULL, "fg_r"),
+		   zcg_float(NULL, "fg_g"),
+		   zcg_float(NULL, "fg_b"),
+		   0.0);
+
+  /* OSD background color */
+  widget = lookup_widget(page, "general-osd-background-selector");
+  gnome_color_picker_set_d(GNOME_COLOR_PICKER(widget),
+		   zcg_float(NULL, "bg_r"),
+		   zcg_float(NULL, "bg_g"),
+		   zcg_float(NULL, "bg_b"),
+		   0.0);
+
+  /* OSD timeout in seconds */
+  widget = lookup_widget(page, "spinbutton2");
+  gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget),
+			    zcg_float(NULL, "timeout"));
+}
+
+static void
+osd_apply		(GtkWidget	*page)
+{
+  GtkWidget *widget;
+  gdouble r, g, b, a;
+
+  widget = lookup_widget(page, "optionmenu22"); /* osd type */
+  zcs_int(z_option_menu_get_active(widget), "osd_type");
+
+  widget = lookup_widget(page, "general-osd-font-selector");
+  zcs_char(gnome_font_picker_get_font_name(GNOME_FONT_PICKER(widget)),
+	   "font");
+
+  widget = lookup_widget(page, "general-osd-foreground-selector");
+  gnome_color_picker_get_d(GNOME_COLOR_PICKER(widget), &r, &g, &b,
+			   &a);
+  zcs_float(r, "fg_r");
+  zcs_float(g, "fg_g");
+  zcs_float(b, "fg_b");
+
+  widget = lookup_widget(page, "general-osd-background-selector");
+  gnome_color_picker_get_d(GNOME_COLOR_PICKER(widget), &r, &g, &b,
+			   &a);
+  zcs_float(r, "bg_r");
+  zcs_float(g, "bg_g");
+  zcs_float(b, "bg_b");
+
+  widget = lookup_widget(page, "spinbutton2"); /* osd timeout */
+  zcs_float(gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget)), "timeout");
+
+}
+
+static void
+add				(GtkDialog *		dialog)
+{
+  SidebarEntry general_options[] = {
+    { N_("OSD"), "gnome-oscilloscope.png",
+      "general-osd-table", osd_setup, osd_apply,
+      .help_link_id = "zapping-settings-osd" }
+  };
+  SidebarGroup groups[] = {
+    { N_("General Options"), general_options, G_N_ELEMENTS (general_options) }
+  };
+
+
+  standard_properties_add (dialog, groups, G_N_ELEMENTS (groups),
+			   "zapping.glade2");
+}
+
 /**
  * Shutdown/startup of the OSD engine
  */
-static gint		input_id = -1;
 ZModel			*osd_model = NULL;
 
 void
@@ -1411,26 +1366,23 @@ startup_osd(void)
 {
   int i;
   GtkWidget *toplevel;
+  property_handler osd_handler = {
+    add: add
+  };
 
   for (i = 0; i<MAX_ROWS; i++)
     matrix[i] = g_malloc0(sizeof(row));
 
-  if (!startup_ucs2())
-    g_warning("Couldn't start Unicode, expect weird things to happen");
-
-  printv("UNICODE: Current locale charset appears to be '%s'\n",
-	 get_locale_charset());
-
+#ifdef HAVE_LIBZVBI
   input_id = gdk_input_add(osd_pipe[0], GDK_INPUT_READ,
 			   osd_event, NULL);
+  CLEAR (osd_page);
+#endif
 
   osd_model = ZMODEL(zmodel_new());
 
-  memset(&osd_page, 0, sizeof(osd_page));
-
   zcc_int(0, "Which kind of OSD should be used", "osd_type");
-  zcc_char("-adobe-times-bold-r-normal-*-14-*-*-*-p-*-iso8859-1",
-	   "Default font", "font");
+  zcc_char("times new roman Bold 36", "Default font", "font");
 
   zcc_float(1.0, "Default fg r component", "fg_r");
   zcc_float(1.0, "Default fg g component", "fg_g");
@@ -1443,12 +1395,19 @@ startup_osd(void)
   zcc_float(1.5, "Seconds the OSD text stays on screen", "timeout");
 
   orphanarium = gtk_fixed_new();
-  gtk_widget_set_usize(orphanarium, 828, 271);
+  gtk_widget_set_size_request (orphanarium, 828, 271);
   toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_container_add(GTK_CONTAINER(toplevel), orphanarium);
   gtk_widget_show(orphanarium);
   gtk_widget_realize(orphanarium);
   /* toplevel will never be shown */
+
+  /* Add Python interface to our routines */
+  cmd_register ("osd_render_markup", py_osd_render_markup, METH_VARARGS);
+  cmd_register ("osd_render", py_osd_render, METH_VARARGS);
+
+  /* Register our properties handler */
+  prepend_property_handler (&osd_handler);
 }
 
 void
@@ -1462,24 +1421,10 @@ shutdown_osd(void)
   for (i = 0; i<MAX_ROWS; i++)
     g_free(matrix[i]);
 
+#ifdef HAVE_LIBZVBI
   gdk_input_remove(input_id);
+#endif
 
-  gtk_object_destroy(GTK_OBJECT(osd_model));
+  g_object_unref (G_OBJECT (osd_model));
   osd_model = NULL;
 }
-
-#else /* !HAVE_LIBZVBI */
-
-#include <gnome.h>
-#include "osd.h"
-
-/* just in case */
-void
-osd_render_sgml		(void (*timeout_cb)(gboolean),
-			 const char *string, ...)
-{
-  if (timeout_cb)
-    timeout_cb(0);
-}
-
-#endif /* !HAVE_LIBZVBI */
