@@ -16,8 +16,11 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#define ZCONF_DOMAIN "/zapping/plugins/"
+#include "zconf.h"
 #include "plugins.h"
 #include "properties.h"
+#include "globals.h"
 
 /* This is the main plugin list, it is only used in plugin_bridge */
 extern GList * plugin_list;
@@ -44,13 +47,11 @@ static gboolean plugin_load(gchar * file_name, struct plugin_info * info)
   gchar * canonical_name;
   gchar * version;
   gint (*plugin_get_protocol)(void) = NULL;
-  gboolean (*plugin_get_symbol)(gchar * name, gint hash,
-				gpointer * ptr);
 
   g_assert(info != NULL);
   g_assert(file_name != NULL);
 
-  memset (info, 0, sizeof(struct plugin_info));
+  CLEAR (*info);
 
   info -> handle = g_module_open (file_name, 0);
 
@@ -71,8 +72,8 @@ static gboolean plugin_load(gchar * file_name, struct plugin_info * info)
   if (((*plugin_get_protocol)()) != PLUGIN_PROTOCOL)
     {
       g_warning("While loading %s\n"
-		"The plugin uses the protocol %d, and the current"
-		" one is %d, it cannot be loaded.",
+		"The plugin uses the protocol %x, and the current"
+		" one is %x, it cannot be loaded.",
 		file_name,
 		(*plugin_get_protocol)(), PLUGIN_PROTOCOL);
       g_module_close(info->handle);
@@ -80,17 +81,17 @@ static gboolean plugin_load(gchar * file_name, struct plugin_info * info)
     }
 
   /* Get the other symbol */
-  if (!g_module_symbol(info->handle, "plugin_get_symbol",
-		       (gpointer*) &(plugin_get_symbol) ))
+  if (!g_module_symbol (info->handle, "plugin_get_symbol",
+			(gpointer *) &info->get_symbol))
     {
-      g_warning(g_module_error());
+      g_warning("%s", g_module_error());
       g_module_close(info->handle);
       return FALSE;
     }
 
   /* plugin_get_info is the only compulsory symbol that must be
      present in the plugin's table of symbols */
-  if (!(*plugin_get_symbol)("plugin_get_info", 0x1234,
+  if (!info->get_symbol("plugin_get_info", 0x1234,
 			    (gpointer*)&(info->plugin_get_info)))
     {
       g_warning("plugin_get_info was not found in %s",
@@ -100,55 +101,55 @@ static gboolean plugin_load(gchar * file_name, struct plugin_info * info)
     }
 
   /* Get the remaining symbols */
-  if (!(*plugin_get_symbol)("plugin_init", 0x1234,
+  if (!info->get_symbol("plugin_init", 0x1234,
 		       (gpointer*)&(info->plugin_init)))
     info->plugin_init = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_close", 0x1234,
+  if (!info->get_symbol("plugin_close", 0x1234,
 		       (gpointer*)&(info->plugin_close)))
     info->plugin_close = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_start", 0x1234,
+  if (!info->get_symbol("plugin_start", 0x1234,
 		       (gpointer*)&(info->plugin_start)))
     info->plugin_start = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_stop", 0x1234,
+  if (!info->get_symbol("plugin_stop", 0x1234,
 		       (gpointer*)&(info->plugin_stop)))
     info->plugin_stop = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_load_config", 0x1234,
+  if (!info->get_symbol("plugin_load_config", 0x1234,
 		       (gpointer*)&(info->plugin_load_config)))
     info->plugin_load_config = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_save_config", 0x1234,
+  if (!info->get_symbol("plugin_save_config", 0x1234,
 		       (gpointer*)&(info->plugin_save_config)))
     info->plugin_save_config = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_running", 0x1234,
+  if (!info->get_symbol("plugin_running", 0x1234,
 		       (gpointer*)&(info->plugin_running)))
     info->plugin_running = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_write_bundle", 0x1234,
-		       (gpointer*)&(info->plugin_write_bundle)))
-    info->plugin_write_bundle = NULL;
+  if (!info->get_symbol("plugin_read_frame", 0x1234,
+		       (gpointer*)&(info->plugin_read_frame)))
+    info->plugin_read_frame = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_read_bundle", 0x1234,
-		       (gpointer*)&(info->plugin_read_bundle)))
-    info->plugin_read_bundle = NULL;
+  if (!info->get_symbol("plugin_capture_start", 0x1234,
+		       (gpointer*)&(info->plugin_capture_start)))
+    info->plugin_capture_start = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_capture_stop", 0x1234,
+  if (!info->get_symbol("plugin_capture_stop", 0x1234,
 		       (gpointer*)&(info->plugin_capture_stop)))
     info->plugin_capture_stop = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_get_public_info", 0x1234,
+  if (!info->get_symbol("plugin_get_public_info", 0x1234,
 		       (gpointer*)&(info->plugin_get_public_info)))
     info->plugin_get_public_info = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_add_gui", 0x1234,
+  if (!info->get_symbol("plugin_add_gui", 0x1234,
 		       (gpointer*)&(info->plugin_add_gui)))
     info->plugin_add_gui = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_remove_gui", 0x1234,
+  if (!info->get_symbol("plugin_remove_gui", 0x1234,
 		       (gpointer*)&(info->plugin_remove_gui)))
     info->plugin_remove_gui = NULL;
 
@@ -157,15 +158,15 @@ static gboolean plugin_load(gchar * file_name, struct plugin_info * info)
       (!info->plugin_remove_gui))
     info -> plugin_add_gui = info -> plugin_remove_gui = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_process_popup_menu", 0x1234,
+  if (!info->get_symbol("plugin_process_popup_menu", 0x1234,
 	    (gpointer*)&(info->plugin_process_popup_menu)))
     info->plugin_process_popup_menu = NULL;
 
-  if (!(*plugin_get_symbol)("plugin_get_misc_info", 0x1234,
+  if (!info->get_symbol("plugin_get_misc_info", 0x1234,
 	    (gpointer*)&(info->plugin_get_misc_info)))
     info->plugin_get_misc_info = NULL;
 
-  memset(&(info->misc_info), 0, sizeof(struct plugin_misc_info));
+  CLEAR (info->misc_info);
 
   if (info -> plugin_get_misc_info)
     memcpy(&(info->misc_info), (*info->plugin_get_misc_info)(),
@@ -180,6 +181,7 @@ static gboolean plugin_load(gchar * file_name, struct plugin_info * info)
 		"provide a canonical name.", file_name);
       return FALSE;
     }
+
   /* Get the version of the plugin */
   version = plugin_get_version(info);
   if (!version)
@@ -222,8 +224,7 @@ static gboolean plugin_load(gchar * file_name, struct plugin_info * info)
 	   realloc(info->exported_symbols,
 		   sizeof(struct plugin_exported_symbol)*(i+1));
 	 if (!info->exported_symbols)
-	  g_error(_("There wasn't enough mem for allocating symbol %d in %s"),
-		  i+1, info->file_name);
+	  g_error(_("Insufficient memory"));
 	
 	info->exported_symbols[i].symbol = g_strdup(symbol);
  	info->exported_symbols[i].type = g_strdup(type);
@@ -232,7 +233,7 @@ static gboolean plugin_load(gchar * file_name, struct plugin_info * info)
 	    g_strdup(description);
 	else
 	  info->exported_symbols[i].description =
-	    g_strdup(_("[No description provided]"));
+	    g_strdup(_("No description available"));
 	info->exported_symbols[i].ptr = ptr;
 	info->exported_symbols[i].hash = hash;
 	info->num_exported_symbols++;
@@ -244,8 +245,6 @@ static gboolean plugin_load(gchar * file_name, struct plugin_info * info)
 
 void plugin_unload(struct plugin_info * info)
 {
-  gint i;
-
   g_assert(info != NULL);
 
   /* Tell the plugin to close itself */
@@ -254,13 +253,6 @@ void plugin_unload(struct plugin_info * info)
   /* Free the memory of the exported symbols */
   if (info -> num_exported_symbols > 0)
     {
-      g_assert(info -> exported_symbols != NULL);
-      for (i=0; i < info->num_exported_symbols; i++)
-	{
-	  g_free(info->exported_symbols[i].symbol);
-	  g_free(info->exported_symbols[i].type);
-	  g_free(info->exported_symbols[i].description);
-	}
       g_free(info -> exported_symbols);
       info -> exported_symbols = NULL;
     }
@@ -269,6 +261,42 @@ void plugin_unload(struct plugin_info * info)
   g_module_close (info -> handle);
 }
 
+struct plugin_info *
+plugin_by_name			(const gchar *		name)
+{
+  GList *glist;
+
+  g_return_val_if_fail (NULL != name, NULL);
+
+  for (glist = g_list_first (plugin_list); glist; glist = glist->next)
+    {
+      struct plugin_info *info;
+
+      info = (struct plugin_info *) glist->data;
+
+      if (0 == strcmp (info->canonical_name, name))
+	return info;
+    }
+
+  return NULL;
+}
+
+gpointer
+plugin_symbol			(const struct plugin_info *info,
+				 const gchar *		name)
+{
+  gpointer ptr;
+
+  g_return_val_if_fail (NULL != info, NULL);
+  g_return_val_if_fail (NULL != name, NULL);
+
+  if (!info->get_symbol (name, 0x1234, &ptr))
+    return NULL;
+
+  return ptr;
+}
+
+/* FIXME: This is ancient, remote.h is much nicer */
 /*
   This is the bridge given to the plugins.
 */
@@ -489,26 +517,22 @@ gboolean plugin_running ( struct plugin_info * info)
   return (*(info->plugin_running))();
 }
 
-void plugin_write_bundle (capture_bundle * bundle, struct plugin_info
-			  * info)
+void plugin_read_frame (capture_frame * frame,
+			struct plugin_info * info)
 {
   g_assert(info != NULL);
-  g_return_if_fail(bundle != NULL);
-  g_return_if_fail(bundle -> image_type != 0);
+  g_return_if_fail(frame != NULL);
 
-  if (info -> plugin_write_bundle)
-    (*info->plugin_write_bundle)(bundle);
+  if (info -> plugin_read_frame)
+    (*info->plugin_read_frame)(frame);
 }
 
-void plugin_read_bundle (capture_bundle * bundle, struct plugin_info
-			 * info)
+void plugin_capture_start (struct plugin_info * info)
 {
   g_assert(info != NULL);
-  g_return_if_fail(bundle != NULL);
-  g_return_if_fail(bundle -> image_type != 0);
 
-  if (info -> plugin_read_bundle)
-    (*info->plugin_read_bundle)(bundle);
+  if (info -> plugin_capture_start)
+    (*info->plugin_capture_start)();
 }
 
 void plugin_capture_stop (struct plugin_info * info)
@@ -558,18 +582,14 @@ void plugin_process_popup_menu (GtkWidget	*widget,
 /* Loads all the valid plugins in the given directory, and appends them to
    the given GList. It returns the new GList. The plugins should
    contain exp in their filename (usually called with exp = .zapping.so) */
-static GList * plugin_load_plugins_in_dir( gchar * directory, gchar * exp,
+static GList * plugin_load_plugins_in_dir( const gchar * directory,
+					   const gchar * exp,
 					   GList * old )
 {
-  struct dirent ** namelist;
-  int n; /* Number of scanned items */
   struct plugin_info plug;
-  struct plugin_info * new_plugin; /* If plugin is OK, a copy will be
-				      allocated here */
-  GList * p;
-
-  int i;
-  gchar * filename; /* Complete path to the plugin to load */
+  GError *error = NULL;
+  GDir *dir;
+  const gchar *name;
 
   g_assert(exp != NULL);
   g_assert(directory != NULL);
@@ -577,29 +597,35 @@ static GList * plugin_load_plugins_in_dir( gchar * directory, gchar * exp,
 
   printv("looking for plugins in %s\n", directory);
 
-  n = scandir(directory, &namelist, 0, alphasort);
-  if (n < 0)
+  dir = g_dir_open (directory, /* flags */ 0, &error);
+
+  if (!dir || error)
     {
-      /* Show error just when there is actually an error */
-      if (errno != ENOENT)
-	perror("scandir");
+      g_assert (!dir);
+
+      if (error)
+	{
+	  printv ("Cannot open directory '%s': %s\n",
+		  directory, error->message);
+	  g_error_free (error);
+	  error = NULL;
+	}
+
       return old;
     }
 
-  for (i = 0; i < n; i++)
+  while ((name = g_dir_read_name (dir)))
     {
-      filename = strstr(namelist[i]->d_name, exp);
+      gchar *filename;
+      GList *p;
+      struct plugin_info *new_plugin;
 
-      if ((!filename) || (strlen(exp) != strlen(filename)))
-	{
-	  free(namelist[i]);
-	  continue;
-	}
+      if (0 != strcmp (name + strlen (name) - strlen (exp), exp))
+	continue;
 
-      filename = g_strconcat(directory, directory[strlen(directory)-1] ==
-			     '/' ? "" : "/", namelist[i] -> d_name,
-			     NULL);
-      free (namelist[i]);
+      filename = g_build_filename (directory, name, NULL);
+
+      printv("loading plugin %s\n", filename);
 
       if (!plugin_load(filename, &plug))
 	{
@@ -607,41 +633,48 @@ static GList * plugin_load_plugins_in_dir( gchar * directory, gchar * exp,
 	  g_free(filename);
 	  continue;
 	}
+
       /* Check whether there is no other other plugin with the same
 	 canonical name */
-      p = g_list_first(old);
-      while (p)
+
+      for (p = g_list_first (old); p; p = p->next)
 	{
-	  new_plugin = (struct plugin_info*)p->data;
-	  if (!strcasecmp(new_plugin->canonical_name,
-			  plug.canonical_name))
+	  new_plugin = (struct plugin_info *) p->data;
+
+	  if (0 == g_ascii_strcasecmp (new_plugin->canonical_name,
+				       plug.canonical_name))
 	    {
 	      /* Collision, load the highest version number */
 	      if (new_plugin->major > plug.major)
-		goto plugin_load_error;
+		{
+		  goto plugin_load_error;
+		}
 	      else if (new_plugin->major == plug.major)
 		{
 		  if (new_plugin->minor > plug.minor)
-		    goto plugin_load_error;
+		    {
+		      goto plugin_load_error;
+		    }
 		  else if (new_plugin->minor == plug.minor)
 		    {
 		      if (new_plugin->micro >= plug.micro)
 			goto plugin_load_error;
 		    }
 		}
+
 	      /* Replace the old one with the new one, just delete the
-	       old one */
-	      old = g_list_remove(old, new_plugin);
-	      plugin_unload(new_plugin);
-	      free(new_plugin);
+		 old one */
+	      old = g_list_remove (old, new_plugin);
+	      plugin_unload (new_plugin);
+	      free (new_plugin);
 	      break; /* There is no need to continue querying */
 	    }
-	  p = p->next;
 	}
-      g_free(filename);
+
+      g_free (filename);
 
       /* This plugin is valid, copy it and add it to the GList */
-      new_plugin = (struct plugin_info *) malloc(sizeof(struct plugin_info));
+      new_plugin = malloc (sizeof (*new_plugin));
       if (!new_plugin)
 	{
 	  perror("malloc");
@@ -649,13 +682,12 @@ static GList * plugin_load_plugins_in_dir( gchar * directory, gchar * exp,
 	  continue;
 	}
 
-      memcpy(new_plugin, &plug, sizeof(struct plugin_info));
+      *new_plugin = plug;
       
       old = g_list_append(old, new_plugin);
     }
 
-  if (namelist)
-    free(namelist);
+  g_dir_close (dir);
 
   return old;
 }
@@ -673,76 +705,41 @@ static gint plugin_sorter (struct plugin_info * a, struct plugin_info * b)
 GList * plugin_load_plugins ( void )
 {
   gchar * plugin_path; /* Path to the plugins */
-  gchar * buffer;
+  const gchar *dir;
+  gchar **plugin_dirs;
   GList * list = NULL;
-  FILE * fd;
   gint i;
 
   /* First load plugins in the home dir */
-  buffer = g_get_home_dir();
-  if (buffer)
+  dir = g_get_home_dir();
+  if (dir)
     {
-      g_assert(strlen(buffer) > 0);
-      if (buffer[strlen(buffer)-1] == '/')
-	plugin_path = g_strconcat(buffer, ".zapping/plugins", NULL);
+      g_assert(strlen(dir) > 0);
+      if (dir[strlen(dir)-1] == '/')
+	plugin_path = g_strconcat(dir, ".zapping/plugins", NULL);
       else
-	plugin_path = g_strconcat(buffer, "/.zapping/plugins", NULL);
+	plugin_path = g_strconcat(dir, "/.zapping/plugins", NULL);
       list = plugin_load_plugins_in_dir (plugin_path, PLUGIN_STRID,
 					 list);
       g_free( plugin_path );
-
-      /* Now load ~/.zapping/plugins_dirs */
-      if (buffer[strlen(buffer)-1] == '/')
-	plugin_path = g_strconcat(buffer, ".zapping/plugin_dirs", NULL);
-      else
-	plugin_path = g_strconcat(buffer, "/.zapping/plugin_dirs",
-				  NULL);
-      fd = fopen (plugin_path, "r");
-      g_free(plugin_path);
-      if (fd)
-	{
-	  buffer = (gchar *) g_malloc0(1024);
-
-	  while (fgets(buffer, 1023, fd))
-	    {
-	      plugin_path = buffer;
-	      /* Skip all spaces */
-	      while (*plugin_path == ' ')
-		plugin_path++;
-
-	      if ((strlen(plugin_path) == 0) || 
-		  (plugin_path[0] == '\n') ||
-		  (plugin_path[0] == '#')) /* Comments and empty lines
-					    */
-		continue;
-	      if (plugin_path[strlen(plugin_path)-1] == '\n')
-		plugin_path[strlen(plugin_path)-1] = 0;
-	      /* Remove all spaces at the end of the string */
-	      i = strlen(plugin_path)-1;
-	      while ((plugin_path[i] == ' ') && (i>=0))
-		{
-		  plugin_path[i--] = 0;
-		}
-	      if (strlen(plugin_path) == 0)
-		continue;
-	      list = plugin_load_plugins_in_dir (plugin_path,
-						 PLUGIN_STRID, list);
-	    }
-	  g_free (buffer);
-	}
     }
 
-  /* Now load plugins in $(prefix)/lib/zapping/plugins */
-  buffer = PACKAGE_LIB_DIR;
+  /* Load plugins in other directories */
+  zcc_char ("",
+	    "Colon separated list of dirs to be searched for plugins",
+	    "plugin_dirs");
+  plugin_dirs = g_strsplit(zcg_char (NULL, "plugin_dirs"), ":", 0);
+  if (plugin_dirs)
+    {
+      for (i=0; plugin_dirs[i]; i++)
+	list = plugin_load_plugins_in_dir (plugin_dirs[i],
+					   PLUGIN_STRID, list);
+      g_strfreev (plugin_dirs);
+    }
 
-  g_assert(strlen(buffer) > 0);
-  if (buffer[strlen(buffer)-1] == '/')
-    plugin_path = g_strconcat(buffer, "plugins", NULL);
-  else
-    plugin_path = g_strconcat(buffer, "/plugins", NULL);
-  /* Load all the plugins in this dir */
-  list = plugin_load_plugins_in_dir (plugin_path, PLUGIN_STRID, list);
-  g_free(plugin_path);
+  /* $(prefix)/lib/zapping/plugins */
+  dir = PACKAGE_LIB_DIR "/plugins";
+  list = plugin_load_plugins_in_dir (dir, PLUGIN_STRID, list);
 
   list = g_list_sort (list, (GCompareFunc) plugin_sorter);
 
@@ -750,7 +747,8 @@ GList * plugin_load_plugins ( void )
 }
 
 /* This function is called from g_list_foreach */
-static void plugin_foreach_free(struct plugin_info * info, void * user_data)
+static void plugin_foreach_free(struct plugin_info * info,
+				void * user_data _unused_)
 {
   plugin_save_config(info);
   plugin_unload(info);
